'''
#---------------------------
# MODULE CONTENTS
#---------------------------
Functions to/for:
1. Calculate general motility features (average speed, distance, ...)
2. Calculate features related to mean squared displacement
3. Calculate features related to random walk similarity

#---------------------------
# To Do
#---------------------------

1. Complete displacement properties statistics
2. Add transforms for time-series analysis of displacements
3. WT-DFA
'''
#------------------------------
# GENERAL MOTILITY STATISTICS
#------------------------------
from __future__ import division
import numpy as np
import math
from scipy import stats
from scipy import interpolate
from dfa import dfa
from statsmodels.tsa.stattools import acf

def average_xy( coor1, coor2 ):
    # Find the average x and y of an XY point
    x1 = float(coor1[0])
    x2 = float(coor2[0])
    y1 = float(coor1[1])
    y2 = float(coor2[1])

    x3 = (x1 + x2) * 0.50
    y3 = (y1 + y2) * 0.50
    return (x3, y3)

def distance( (x1,y1), (x2,y2) ):
    # Calculates euclidean distance
    d = math.sqrt( (x2-x1)**2.00 + (y2-y1)**2.00)
    return d

class GeneralFeatures:
    def __init__(self, cell_ids, move_thresh = 10):
        self.cell_ids = cell_ids
        if len(cell_ids[cell_ids.keys()[0]]) > 19:
            self.moving_tau = move_thresh
            self.tau_range = range(9,12)
            self.interval_range = range(5,7)
        elif 9 < len(cell_ids[cell_ids.keys()[0]]) <= 19:
            self.moving_tau = move_thresh
            self.tau_range = range(6,7)
            self.interval_range = range(3,4)
        else:
            print "Time series too small to calculate turning features"
        self.total_distance, self.avg_speed, self.d = self.calc_total_distance(self.cell_ids)
        self.net_distance = self.calc_net_distance(self.cell_ids)
        self.linearity = self.calc_linearity(self.cell_ids)
        self.spearmanrsq = self.calc_spearman(self.cell_ids)
        self.progressivity = self.calc_progressivity(self.cell_ids, net_distance=self.net_distance, total_distance=self.total_distance)
        self.min_speed, self.max_speed = self.calc_minmax_speed(self.cell_ids)
        self.moving_tau = 5
        self.avg_moving_speed, self.time_moving = self.calc_moving_variable_threshold(cell_ids, thresholds = range(1,11), tau = self.moving_tau)

        self.turn_stats, self.theta_stats = self.all_turn_stats(cell_ids, tau_range = self.tau_range, interval_range = self.interval_range)

    # Calculate total distance traveled and save each cell's stat
    # as a float in a list total_distance
    # Calculates distance b/w each pair of points, adds to a temp
    # list traveled and then takes the sum as the total distance
    # Also calculates average speed in px/hour as mean_speed
    def calc_total_distance(self, cell_ids):
        total_distance = {}
        avg_speed = {}
        distance_dist = {}
        for u in cell_ids:
            cell = cell_ids[u]
            start_point = cell[0]
            traveled = []
            for coor in cell[1:]:
                d = distance( start_point, coor )
                traveled.append(d)
                start_point = coor
            total_distance[u] = sum(traveled)
            avg_speed[u] = np.mean(traveled)
            distance_dist[u] = traveled

        return total_distance, avg_speed, distance_dist

    # Calculate net distance traveled and save each cell's stat
    # as a float in a list net_distance
    # Simple distance b/w start and end points
    def calc_net_distance(self, cell_ids):
        net_distance = {}
        for u in cell_ids:
            cell = cell_ids[u]
            start_point = cell[0]
            end_point = cell[-1]
            d = distance( start_point, end_point)
            net_distance[u] = d
        return net_distance

    # Calculate linearity of the traveled path as the R^2 value
    # of a linear regression of all path points
    def calc_linearity(self, cell_ids):
        linearity = {}
        for u in cell_ids:
            cell = cell_ids[u]
            x = []
            y = []
            for coor in cell:
                x.append( coor[0] )
                y.append( coor[1] )
            slope, intercept, r_value, p_value, std_err = stats.linregress(x,y)
            r_squared = r_value**2
            linearity[u] = r_squared
        return linearity

    # Calculate Spearman's rho^2 for all XY points in a cell path
    def calc_spearman(self, cell_ids):
        spearmanrsq = {}
        for u in cell_ids:
            xy = np.array( cell_ids[u] )
            rho, pval = stats.spearmanr( xy )
            spearmanrsq[u] = rho**2
        return spearmanrsq

    def calc_progressivity(self, cell_ids, net_distance, total_distance):
        progressivity = {}
        for u in cell_ids:
            progressivity[u] = float(net_distance[u]) / float(total_distance[u])

        return progressivity

    def calc_minmax_speed(self, cell_ids):
        '''
        Calculate max/min speeds as the distance traveled
        across 5 frames and append as a float to list max_speed/min_speed
        Speeds in the unit of pixels per hour
        '''
        max_speed = {}
        min_speed = {}
        for u in cell_ids:
            cell = cell_ids[u]
            speeds = []
            i = 0
            while (i + 4) < len(cell):
                d = distance( cell[i], cell[i+4] )
                speeds.append( d / 5 )
                i += 1
            max_speed[u] = max(speeds)
            min_speed[u] = min(speeds)
        return min_speed, max_speed

    def calc_moving_stats(self, cell_ids, move_speed, tau):
        '''
        Calculates the proportion of time a cell spends moving_speeds,
        min, max, and average speeds while moving

        Parameters
        ------------
        cell_ids : dict keyed by cell id, containing lists of tupled XY coors
        move_speed : minimum speed to be considered "moving"
        tau : time lag to use when determining movement speed
        '''
        avg_moving_speed = {}
        time_moving = {}
        for u in cell_ids:
            cell = cell_ids[u]
            speeds = []
            i = 0
            while (i + tau) < len(cell):
                d = distance( cell[i], cell[i+tau] )
                speeds.append( d / tau )
                i += 1
            moving_speeds = []
            for val in speeds:
                if val > move_speed:
                    moving_speeds.append( val )
                else:
                    pass

            time_moving[u] = float(len(moving_speeds)) / float(len(speeds))

            if moving_speeds == []:
                moving_speeds.append(0)
            else:
                pass
            avg_moving_speed[u] = np.mean( moving_speeds )

        return avg_moving_speed, time_moving

    def calc_moving_variable_threshold(self, cell_ids, thresholds = range(1,11), tau = 5):

        # dicts keyed by moving speed threshold used for calc
        # each key corresponds to a dict keyed by cell_id, with vals for moving speed
        # and time moving for the given threshold in the top level dict
        avg_moving_speed = {}
        time_moving = {}

        for thresh in thresholds:
            avg_moving_speed[thresh], time_moving[thresh] = self.calc_moving_stats( cell_ids, thresh, tau )

        return avg_moving_speed, time_moving

    #------------------------------
    # TURNING STATISTICS
    #------------------------------

    def turn_direction(self, cell_path, tau = 10, interval = 5):
        '''
        Returns the proportion of time a cell turns left as float 0.0-1.0

        Parameters
        ------------
        cell_path : list containing a timeseries of tuples,
                    each tuple holding an XY position
                    i.e. cell_path = [(x1,y1), (x2,y2)...(xn,yn)]
        tau       : integer indicating the desired time lag between a point of
                    interest and a point in the distance (p_n+tau) to determine
                    a cell's turning behavior. Must be > interval
        interval  : integer representing the number of points ahead and behind
                    the point of interest to consider when find a regression
                    line to represent the cell's direction at p_n

        Returns
        ------------
        turns     : tuple of binaries reflecting turns left (0) or right (1)
        thetas       : angles of each turn in radians


        Principle
        ------------
        (1) Estabish direction of object at p_n:
        Given a time series XY of x,y coordinates, where p_t denotes a point
        at time t, take a given point p_n
        determine the 'direction' of motion at p_n by plotting a linear reg.
        on points [p_n-i, p_n+i] for some interval value i
        This linear regression function is R(x) = slope*x + b

        (2) Determine if p_n+tau is left or right
        For a point p_n+tau, for a variable time lag tau, determine if p_n+tau
        lies left or right of R
            p_n+tau,Y > R(p_n+tau) and initial direction = right : left turn
            p_n+tau,Y < R(p_n+tau) and initial direction = right : right turn
            ...

        (3) Calculate magnitude of turn theta
        Calculate the angle of the turn theta as the angle between
            R(x) from (1)
            R2, line connecting p_n to p_n+tau

            v1 = (1, slope of R(x))
            v2 = (1, slope of R2)

            theta = arccos( v1 dot v2 / ||v1|| * ||v2||)

        Notable Behavior
        ------------
        If a cell moves in a perfectly straight line, such that point of
        interest p_n has x coordinate == p_n+tau, the function skips this p_n
        i.e. Skips if the cell has moved in a perfectly linear line, or circled
        back on itself

        '''
        if tau < interval:
            print 'tau must be > interval'
            return None
        else:
            pass

        XY = np.asarray(cell_path)

        turns = []
        thetas = []
        # do every 3rd point to cut runtime
        for n in range( interval, len(XY)-tau ):
            if len(XY) > 30 and n % 3 != 0:
                continue
            p_n = XY[n]
            p_tau = XY[n+tau]
            if p_n[0] == p_tau[0]:
                continue
            direction = p_tau[0] - p_n[0]
            regress_p = XY[n-interval:n+interval]
            # skip p_n if all x coors in interval are equal
            if all(regress_p[:,0] == regress_p[0,0]):
                continue
            R_slope, b, rval, pval, stderr = stats.linregress(regress_p[:,0], regress_p[:,1])
            # skip if cell moves entirely on a vertical line for given interval period
            if np.isnan(R_slope):
                continue
            R_tau = R_slope*(p_tau[0]) + b
            # direction > 0 = +x movement, < 0 = -x
            # turn = 0 : left
            # turn = 1 : right
            if direction > 0:
                if R_tau < p_tau[1]:
                    turn = 0
                else:
                    turn = 1
            else:
                if R_tau < p_tau[1]:
                    turn = 1
                else:
                    turn = 0
            turns.append(turn)

            # find line R2 connecting p_n and p_tau
            R2_slope = (p_tau[1] - p_n[1]) / (p_tau[0] - p_n[0])
            R2_intercept = p_n[1] - R2_slope*p_n[0]
            # create vectors v1 : R, v2 : R2
            v1 = np.array( [1, R_slope] )
            v2 = np.array( [1, R2_slope] )
            # cos(theta) = v1 dot v2 / ||v1|| * ||v2||
            if (np.linalg.norm(v1)*np.linalg.norm(v2)) == 0:
                print "error (np.linalg.norm(v1)*np.linalg.norm(v2)) == 0"
                print n, p_n, p_tau
                continue
            costheta = np.dot(v1, v2) / (np.linalg.norm(v1)*np.linalg.norm(v2))
            #print "v1 = ", v1, "| v2 = ", v2, "| costheta = ", costheta
            if costheta < -1.0 or costheta > 1.0:
                #print "Error calculating turn magnitude"
                #print "costheta = ", costheta
                #print "v1, v2 = ", v1, v2
                continue
            theta = math.acos(costheta)
            thetas.append(theta)

        return turns, thetas

    def all_turn_stats( self, cell_ids, tau_range = range(9,12), interval_range = range(5,7)):
        '''
        Parameters
        ----------
        cell_ids       : dict of lists containing tuples, each tuple an xy point at
                         a given timepoints
        tau_range      : range of time lags to try for calculation
        interval_range : range of interval distances to use for calculation of
                         the linregress about point of interest p_n

        Returns
        ----------
        turn_stats     : dict keyed by tau, containing dict keyed by interval,
                         containing dict keyed by cell, containing proportion of
                         time a cell turns right
        theta_stats    : dict keyed by tau, containing dict keyed by interval,
                         containing dict keyed by cell, containing a list of
                         three elements [avg_theta, min_theta, max_theta]
        '''
        turn_stats = {}
        theta_stats = {}
        for tau in tau_range:
            turn_stats[tau] = {}
            theta_stats[tau] = {}
            for interval in interval_range:
                turn_stats[tau][interval] = {}
                theta_stats[tau][interval] = {}
                for u in cell_ids:
                    cell_path = cell_ids[u]
                    turns, thetas = self.turn_direction(cell_path, tau = tau, interval = interval)
                    turn_stats[tau][interval][u] = np.mean(turns)
                    # calc theta stats, set as val in triple dict
                    ts = [np.mean(thetas), min(thetas), max(thetas)]
                    theta_stats[tau][interval][u] = ts
        return turn_stats, theta_stats



#--------------------------------
#     MEAN SQUARE DISPLACEMENT
#--------------------------------

# MSD(tau) = (1 / tau) * sum( (x(t + tau) + x(t))^2 )
# Plotting a distribution of Tau vs MSD will generate a curve
# The exponential nature of this curve will describe the type
# of motion the particle is experiencing
# 1 < alpha --> impeded diffusion
# Linear curve (alpha = 1) --> diffusion
# 1 < alpha < 2 --> super diffusion
# alpha = 2 --> ballistic motion

class MSDFeatures:
    def __init__(self, cell_ids):
        self.msd_distributions, self.log_distributions = self.msd_dist_all_cells(cell_ids, tau = 31)
        self.alphas = self.calc_alphas(self.log_distributions)

    # Calculate the mean square displacement for a given tau value
    # across all timepoints of a tracked object
    def calc_msd(self, path, tau):
        '''
        Calculates mean squared displacement for a cell path
        for a given time lag tau

        Parameters
        -------------------
        path : list containing tuples of sequential XY coordinates
        tau  : time lag to consider when calculating MSD

        Returns
        -------------------
        msd : mean squared displacement of path given time lag tau
        '''
        distances = []
        t = 0
        while (t + tau) < len(path):
            distsq = ( distance(path[ t + tau ], path[ t ]) )**2
            distances.append(distsq)
            t += 1

        msd = sum(distances)/t

        return msd

    # Calculate the MSD distribution for a given range of tau's
    def calc_msd_distribution(self, path, max_tau):
        '''
        Calculates the distribution of MSDs for a range of time
        lag values tau

        Parameters
        -------------------
        path : list of tuples containing sequential XY coordinates
        max_tau : maximum time lag to consider for MSD calculation

        Returns
        -------------------
        distribution : MSDs in a list, indexed by tau
        log_distribution : log transform of distribution

        Notable Behavior
        -------------------
        If a cell is stationary indefinitely, it effectively has
        a MSD of 0
        However, this calls a math domain error, so checks are in
        place that ensure the final 'alpha' calculation will be
        0 without raising exceptions

        Here -- returns both distributions as a string 'flat' if
        MSD calc is 0 for a given range tau
        '''
        tau = 1
        distribution = []
        log_distribution = []
        while tau < max_tau:
            msd = self.calc_msd(path, tau)
            distribution.append(msd)
            if msd > 0:
                log_distribution.append( math.log(msd) )
            else:
                distribution = 'flat'
                log_distribution = 'flat'
                return distribution, log_distribution
            tau += 1

        return distribution, log_distribution

    # Calculate the MSD distribution for all cells in the cell_ids
    # dicts. Return distributions and log distributions in seperate dicts
    def msd_dist_all_cells(self, cell_ids, tau = 31):
        '''
        Calculates MSD distributions for all cells in dict cell_ids

        Parameters
        -------------------
        cell_ids : dictionary keyed by cell_id, contains lists of
                   tuples of sequential XY coordinates
        tau      : maximum time lag to calculate for MSD distributions
                   n.b. if path < 30 t's long, uses len(path) as max tau

        Returns
        -------------------
        msd_distributions : dict keyed by cell_id containing list
                            of MSD distributions, indexed by tau
        log_distributions : dict with log transformed lists of
                            MSD distributions

        Notable behavior
        -------------------
        checks for distributions == 'flat' string and sets values for
        that cell as 'flat', which is checked to provide an alpha of 0

        '''

        msd_distributions = {}
        log_distributions = {}
        for u in cell_ids:
            cell = cell_ids[u]
            if len(cell) > 30:
                max_tau = tau
            else:
                max_tau = len(cell)
            dist, log_dist = self.calc_msd_distribution(cell, max_tau)
            if dist == 'flat':
                msd_distributions[u] = 'flat'
                log_distributions[u] = 'flat'
            else:
                msd_distributions[u] = dist
                log_distributions[u] = log_dist

        return msd_distributions, log_distributions

    def calc_alphas(self, log_distributions):
        '''
        Calculate the alpha coefficient value as the slope of a
        log(MSD) v log(tau) plot

        Parameters
        -----------------
        log_distributions : dict keyed by cell_id containing lists
        of log transformed MSDs, indexed by tau

        Returns
        ----------------
        alphas : dict keyed by cell_id, with values as the alpha coeff
                 from log(MSD(tau)) = alpha*log(tau)

        Notable Behavior
        ----------------
        Checkes if log_distributions[cell_id] is == 'flat'
        If so, sets alpha at the appropriate "flat" slope of 0
        '''

        alphas = {}

        for u in log_distributions:
            if log_distributions[u] == 'flat':
                alphas[u] = 0
            else:
                tau = np.arange(1, len(log_distributions[u]) + 1 )
                log_tau = np.log(tau)

                slope, intercept, r_val, p_val, SE = stats.linregress( log_tau, log_distributions[u] )
                alphas[u] = slope

        return alphas

    # Plots tau vs MSD and log(tau) vs log(MSD)
    # Saves as a PDF in the motility_statistics output folder
    def plot_msd_dists(self, output_dir, msd_distributions, log_distributions):

        tau = range(1,31)
        log_tau = []
        for val in tau:
            log_tau.append( math.log(val) )

        plt.figure(1)
        plt.subplot(121) # call subplots, 1 row, 2 columns, plot 1
        for u in msd_distributions:
            plt.scatter(tau, msd_distributions[u])
            plt.plot(tau, msd_distributions[u])
            plt.xlabel(r'$\tau$') # raw text string, latex to call tau char
            plt.ylabel('Mean Square Displacement')

        plt.subplot(122) # call subplots, 1 row, 2 columns, plot 2
        for u in msd_distributions:
            plt.scatter(log_tau, log_distributions[u])
            plt.plot(log_tau, log_distributions[u])
            plt.xlabel(r'log($\tau$)')
            plt.ylabel('log(Mean Square Displacement)')

        plt.subplots_adjust(wspace = 0.2) # increasing spacing b/w subplots

        plt.savefig(str(output_dir + 'MSD_Plots.pdf'))

#--------------------------------
# RANDOM WALK MODELING
#--------------------------------

class RWFeatures:
    '''
    Calculates features:
    * comparing properties of a cell's displacement distribution to RWs vs.
      more long-tailed distributions, indicative of Levy flight like motion
    * comparing a cell to random walk simulations of motion
    * estimating self-similarity metrics

    '''
    def __init__(self, cell_ids, gf): # gf = GeneralFeatures
        self.diff_linearity, self.diff_net_dist = self.run_comparative_randwalk(cell_ids, gf.linearity, gf.net_distance, gf.avg_speed)
        self.cell_kurtosis, self.diff_kurtosis = self.vartau_kurtosis_comparison(cell_ids, max_tau = 10)
        self.nongaussalpha = self.nongauss_coeff(cell_ids)
        self.disp_var, self.disp_skew = self.displacement_props(cell_ids)
        self.hurst_RS = self.hurst_mandelbrot(cell_ids)
        self.autocorr_max_tau = 11
        self.autocorr, self.qstats, self.pvals = self.autocorr_all(cell_ids, max_tau = self.autocorr_max_tau)
        #self.hurst_dfa = self.dfa_all(cell_ids)

    # Creates a random walk model, given an origin point, N number of steps
    # and a defined rate of motion per step
    # Exports list of the model_path, values for model net distance & linearity
    def random_walk(self, origin, N, speed_mu, speed_sigma = None):
        import numpy as np
        '''
        Net distance of random walks is based on
        tau = time_unit
        rate = sigma = step_size = time_unit * velocity_x
        n = number of steps (can be exchanged for t if =)
        <x(t)^2> = n * sigma^2
        <x(t)^2>^0.5 = sigma * sqrt(n) = step_size * sqrt(n)
        <(x,y)^2> = <r^2> = <x(t)^2> + <y(t)^2> = 2 * n * sigma^2
        <r^2>^0.5 = sqrt(2) * step_size * sqrt(n)

        step_size = rate = c = sqrt(x_step^2 + y_step^2) = sqrt(2*step^2)
        x_step = y_step = rate/sqrt(2)
        (Random Walks in Biology, 1992, Howard C. Berg)

        Parameters
        ------------------
        origin : starting point for the model
        N      : number of steps to model
        speed_mu : mean speed for the model
        speed_sigma : stdev of the normal distribution for step sizes
        '''
        model_net_dist = math.sqrt(2) * speed_mu * math.sqrt( N )
        model_path = [ origin ]
        if speed_sigma == None:
            speed_sigma = 0.2*speed_mu
        rate = round( np.random.normal(speed_mu, speed_sigma), 3)
        vectors = [ (0, rate), (0, -rate), (rate, 0), (-rate, 0) ]
        i = 0
        while i < N:
            walk = np.random.random()
            if 0 <= walk < 0.25:
                step = vectors[0]
            elif 0.25 <= walk < 0.50:
                step = vectors[1]
            elif 0.50 <= walk < 0.75:
                step = vectors[2]
            else:
                step = vectors[3]

            new_x = model_path[-1][0] + step[0]
            new_y = model_path[-1][1] + step[1]
            model_path.append( (new_x, new_y) )
            i += 1


        x = []
        y = []
        for coor in model_path:
            x.append( coor[0] )
            y.append( coor[1] )

        try:
            slope, intercept, r_value, p_value, std_err = stats.linregress(x,y)
            model_linearity = r_value**2
        except:
            return 'error', 'error', 'error'

        return model_net_dist, model_linearity, model_path

    # Where cell is a list of (x,y) tuple-coordinates denoting the cell path
    # u is the key of the cell in all data dicts
    # executes a random walk model with the cell's given rate and timescale
    # returns differences between the model and the actual cell
    def compare_to_randwalk(self, cell, u, cell_linearity, cell_net_dist, cell_avg_speed):
        origin = cell[0]
        N = len(cell)
        rate = cell_avg_speed
        linearitys = []

        i = 0
        while i < 200:
            model_net_dist, model_linearity, model_path = self.random_walk(origin, N, rate)
            if model_net_dist == 'error':
                pass
            else:
                linearitys.append( model_linearity )
            i += 1

        avg_linearity = np.mean(linearitys)

        diff_linearity = cell_linearity - avg_linearity
        diff_net_dist = cell_net_dist - model_net_dist

        return diff_linearity, diff_net_dist

    def run_comparative_randwalk(self, cell_ids, linearity, net_distance, avg_speed):
        diff_linearity = {}
        diff_net_dist = {}
        for u in cell_ids:
            cell = cell_ids[u]
            lin, net = self.compare_to_randwalk(cell, u, linearity[u], net_distance[u], avg_speed[u])
            diff_linearity[u] = lin
            diff_net_dist[u] = net

        return diff_linearity, diff_net_dist

    # Makes a list containing cell displacements for a variable time scale, tau
    def make_displacement_dist(self, cell_ids, tau):

        displacement_dist = {}
        for u in cell_ids:
            cell = cell_ids[u]
            displacement_dist[u] = []
            i = 1
            while (i + tau) < len(cell):
                d = distance( cell[i + tau], cell[i] )
                displacement_dist[u].append( d )
                i = i + tau

        return displacement_dist

    def kurtosis_comparison(self, cell_ids, displacements):

        cell_kurtosis = {}
        diff_kurtosis = {}
        rayleigh_kurtosis = 3.245089
        for u in cell_ids:
            dist = displacements[u]
            cell_kurtosis[u] = stats.kurtosis(dist)
            diff_kurtosis[u] = cell_kurtosis[u] - rayleigh_kurtosis

        return cell_kurtosis, diff_kurtosis

    def vartau_kurtosis_comparison(self, cell_ids, max_tau = 10 ):
        '''
        calculate cell_kurtosis and diff_kurtosis for a range of possible
        time intervals (tau) from cell_ids and a list of tau
        Outputs dict of dicts, keyed by tau, containing dicts keyed by cell_id
        values of which are cell_kurtosis and diff_kurtosis
        '''
        # set up dicts keyed by tau, containing dicts keyed by cell_id
        all_cell_kurtosis = {}
        all_diff_kurtosis = {}

        tau = 1
        while tau <= max_tau:
            displacement_dist = self.make_displacement_dist(cell_ids, tau)
            all_cell_kurtosis[tau], all_diff_kurtosis[tau] = self.kurtosis_comparison(cell_ids, displacements = displacement_dist)
            tau += 1

        return all_cell_kurtosis, all_diff_kurtosis

    def displacement_props(self, cell_ids):
        '''
        Calculates variance and skewness of the displacement distribution
        for each cell in cell_ids

        Parameters
        -----------------
        cell_ids : dict of lists containing tuples of sequential XY coordinates

        Returns
        -----------------
        var : dict keyed by cell_id with variance of displacement distribution
        skew : dict keyed by cell_ids with skew of displacement distribution
        '''
        var = {}
        skew = {}

        allX = self.make_displacement_dist(cell_ids=cell_ids, tau = 1)
        for u in allX:
            X = np.asarray( allX[u] )
            var[u] = np.var(X)
            skew[u] = stats.skew(X)

        return var, skew

    def calc_ngaussalpha(self, X):
        '''
        Calculates the non-Gaussian parameter alpha_2 of a given displacement
        distribution, X

        alpha_2 = <dx^4> / 3*<dx^2>^2 - 1

        effectively, a ratio of coeff* (kurtosis / variance)

        For a Gaussian distribution, alpha_2 = 0
        For non-Gaussian distributions, alpha_2 increases with the length
        of the tails

        Levy-like motion would be expected to have alpha_2 > 0, while diffusive
        motion would be expected to have alpha_2 == 0

        Rayleigh alpha_2 ~= -0.33

        Parameters
        ----------------------
        X : distribution of displacements, either np.array or list

        Returns
        ----------------------
        alpha_2 : non-Gaussian coefficient, floating point values
        '''

        X = np.asarray(X)
        alpha_2 = np.mean( X**4 ) / (3*np.mean(X**2)**2) - 1

        return alpha_2

    def nongauss_coeff(self, cell_ids):
        '''
        Calculates non-Gaussian coefficient alpha_2 for all cells in cell_ids

        Parameters
        ----------------------
        cell_ids : dict of lists containing tuples of sequential XY coordinates

        Returns
        ----------------------
        nongauss_coeff : dict keyed by cell_id, nonGauss coeff values
        '''

        ngaussalpha = {}
        allX = self.make_displacement_dist(cell_ids=cell_ids, tau=1)
        for u in allX:
            X = allX[u]
            ngaussalpha[u] = self.calc_ngaussalpha(X)

        return ngaussalpha


    def largest_pow2(self, num):
        for i in range(0,10):
            if int(num / 2**i) > 1:
                continue
            else:
                return i-1

    def rescaled_range(self, X, n):
            # takes a series X and subseries size n
            # finds the average rescaled range <R(n)/S(n)>
            # for all sub-series of size n
            N = len(X)
            if n > N:
                return None
            # Create subseries of size n
            num_subseries = int(N/n)
            Xs = np.zeros((num_subseries, n))
            for i in range(0, num_subseries):
                Xs[i,] = X[ int(i*n) : int(n+(i*n)) ]

            # Calculate mean rescaled range R/S
            # for subseries size n
            RS = []
            for subX in Xs:

                m = np.mean(subX)
                Y = subX - m
                Z = np.cumsum(Y)
                R = max(Z) - min(Z)
                S = np.std(subX)
                if S <= 0:
                    print "S = ", S
                    continue
                RS.append( R/S )
            RSavg = np.mean(RS)

            return RSavg

    def hurst_mandelbrot(self, cell_ids):
        '''
        for E[R(n)/S(n)] = Cn**H as n --> inf
        H : 0.5 - 1 ; long-term positive autocorrelation
        H : 0.5 ; fractal Brownian motion
        H : 0-0.5 ; long-term negative autocorrelation

        N.B. SEQUENCES MUST BE >= 18 units long, otherwise
        linear regression for log(R/S) vs log(n) will have
        < 3 points and cannot be performed
        '''

        hurst_RS = {}
        allX = self.make_displacement_dist(cell_ids, tau = 1)
        for u in allX:
            X = allX[u]
            RSl = []
            ns = []
            for i in range(0,self.largest_pow2(len(X))):
                ns.append(int(len(X)/2**i))
            for n in ns:
                RSl.append( self.rescaled_range(X, n) )
            m, b, r, pval, sderr = stats.linregress(np.log(ns), np.log(RSl))
            hurst_RS[u] = m
        return hurst_RS

    # using Dominik Krzeminski's (@dokato) dfa library
    # see
    # https://github.com/dokato/
    # https://github.com/dokato/dfa/blob/master/dfa.py
    from dfa import dfa
    def dfa_all(self, cell_ids):
        '''
        Performs detrended fluctuation analysis on cell displacement series,
        as described in
        Ping et. al., Mosaic organization of DNA nucleotides, 1994, Phys Rev E

        Parameters
        ----------------------
        cell_ids : dict of lists keyed by cell_id
        ea. list represents a cell. lists contain sequential tuples
        containing XY coordinates of a cell at a given timepoint

        Returns
        ----------------------
        dfa_alpha : dict keyed by cell_id, values are the alpha coefficient
        calculated by detrended fluctuation analysis
        '''
        dfa_alpha = {}
        allX = self.make_displacement_dist(cell_ids, tau = 1)
        for u in allX:
            X = allX[u]
            scales, fluct, coeff = dfa(X, scale_lim = [2,7], scale_dens = 0.25)
            dfa_alpha[u] = coeff
        return dfa_alpha

    def autocorr_all(self, cell_ids, max_tau = 10):
        '''
        Estimates the autocorrelation coefficient for each series of cell
        displacements over a range of time lags.
        Estimation method:
        https://en.wikipedia.org/wiki/Autocorrelation#Estimation

        R(tau) = 1/(n-tau)*sigma**2 [sum(X_t - mu)*(X_t+tau - mu)] | t = [1,n-tau]

        X as a time series, mu as the mean of X, sigma**2 as variance of X
        tau as a given time lag (sometimes referred to as k in literature)

        Implementation uses statsmodels.tsa.stattools.acf()

        Parameters
        ----------------------
        cell_ids : dict of lists keyed by cell_id
        ea. list represents a cell. lists contain sequential tuples
        containing XY coordinates of a cell at a given timepoint

        Returns
        ----------------------
        autocorr : dict of lists, containing autocorrelation coeffs for
        sequential time lags
        qstats : dict of lists containing Q-Statistics (Ljung-Box)
        pvals : dict of lists containing p-vals, as calculated from Q-Statistics

        n.b. truncated to taus [1,10], to expand to more time lags, simply
        alter the indexing being loaded into the return dicts
        '''
        autocorr = {}
        qstats = {}
        pvals = {}
        allX = self.make_displacement_dist(cell_ids, tau = 1)
        for u in allX:
            X = allX[u]
            # Perform autocorrelation, lages [0,29] tested
            # Perform Ljung-Box Q-statistic calculation to determine if
            # autocorrelations detected are significant or random
            #
            ac, q, p = acf(X, unbiased = True, nlags = (max_tau+1), qstat=True)
            autocorr[u] = ac[1:max_tau]
            qstats[u] = q[1:max_tau]
            pvals[u] = p[1:max_tau]
        return autocorr, qstats, pvals



import numpy as np
import scipy.signal as ss

# detrended fluctuation analysis

def calc_rms(x, scale):
    """
    Root Mean Square in windows with linear detrending.

    Args:
    -----
      *x* : numpy.array
        one dimensional data vector
      *scale* : int
        length of the window in which RMS will be calculaed
    Returns:
    --------
      *rms* : numpy.array
        RMS data in each window with length len(x)//scale
    """
    # making an array with data divided in windows
    shape = (x.shape[0]//scale, scale)
    X = np.lib.stride_tricks.as_strided(x,shape=shape)
    # vector of x-axis points to regression
    scale_ax = np.arange(scale)
    rms = np.zeros(X.shape[0])
    for e, xcut in enumerate(X):
        coeff = np.polyfit(scale_ax, xcut, 1)
        xfit = np.polyval(coeff, scale_ax)
        # detrending and computing RMS of each window
        rms[e] = np.sqrt(np.mean((xcut-xfit)**2))
    return rms

def dfa(x, scale_lim=[5,9], scale_dens=0.25, show=False):
    """
    Detrended Fluctuation Analysis - algorithm with measures power law
    scaling of the given signal *x*.
    More details about algorithm can be found e.g. here:
    Hardstone, R. et al. Detrended fluctuation analysis: A scale-free
    view on neuronal oscillations, (2012).

    Args:
    -----
      *x* : numpy.array
        one dimensional data vector
      *scale_lim* = [5,9] : list of lenght 2
        boundaries of the scale where scale means windows in which RMS
        is calculated. Numbers from list are indexes of 2 to the power
        of range.
      *scale_dens* = 0.25 : float
        density of scale divisions
      *show* = False
        if True it shows matplotlib picture
    Returns:
    --------
      *scales* : numpy.array
        vector of scales
      *fluct* : numpy.array
        fluctuation function
      *alpha* : float
        DFA exponent
    """
    # cumulative sum of data with substracted offset
    y = np.cumsum(x - np.mean(x))
    scales = (2**np.arange(scale_lim[0], scale_lim[1], scale_dens)).astype(np.int)
    fluct = np.zeros(len(scales))
    # computing RMS for each window
    for e, sc in enumerate(scales):
        fluct[e] = np.mean(np.sqrt(calc_rms(y, sc)**2))
    # fitting a line to rms data
    coeff = np.polyfit(np.log2(scales), np.log2(fluct), 1)
    if show:
        fluctfit = 2**np.polyval(coeff,np.log2(scales))
        plt.loglog(scales, fluct, 'bo')
        plt.loglog(scales, fluctfit, 'r', label=r'$\alpha$ = %0.2f'%coeff[0])
        plt.title('DFA')
        plt.xlabel(r'$\log_{10}$(time window)')
        plt.ylabel(r'$\log_{10}$<F(t)>')
        plt.legend()
        plt.show()
    return scales, fluct, coeff[0]


if __name__=='__main__':
    n = 1000
    x = np.random.randn(n)
    # computing DFA of signal envelope
    x = np.abs(ss.hilbert(x))
    scales, fluct, alpha = dfa(x, show=1)
    print scales
    print fluct
    print "DFA exponent: {}".format(alpha)



import csv
import glob
import pickle

'''
#---------------------------
# MODULE CONTENTS
#---------------------------
Functions to/for:
Import/export data

'''

#------------------------------
# IMPORT CENTROIDS FROM CSVs
#------------------------------


def import_centroids(input_dir):
    print "--------------------------------"
    print input_dir
    print "--------------------------------"
    centroid_arrays = []
    for matlab_export in glob.glob( input_dir ):
        in_file = open(matlab_export, "rU")
        with in_file as f:
            reader = csv.reader(f, delimiter=',')
            x, y = zip(*reader)

        i = 0
        array = []
        while i < len(x):
            array.append( ( float(x[i]), float(y[i]) ) )
            i += 1
        centroid_arrays.append(array)
    return centroid_arrays

#------------------------------
# WRITE STATISTICS TO CSV
#------------------------------

def write_prop_colliders(prop_colliders, output_dir):

    out_file = (output_dir + 'prop_colliders.csv')
    with open(out_file, 'wb') as f:
        selectric = csv.writer(f, delimiter = ',', quoting=csv.QUOTE_NONE, quotechar='')
        selectric.writerow( [prop_colliders] )

def write_motility_stats(output_dir, output_name, gf, rwf, merged_list):
    output_file_path = str(output_dir + output_name)

    motility_header = [ 'Well/XY', 'cell_id', 'total_distance', 'net_distance', 'linearity', 'spearmanrsq','progressivity',
    'max_speed', 'min_speed', 'avg_speed', 'MSD_slope', 'hurst_RS', 'nongauss', 'disp_var', 'disp_skew', 'rw_linearity', 'rw_netdist', 'rw_kurtosis01',
    'rw_kurtosis02', 'rw_kurtosis03', 'rw_kurtosis04', 'rw_kurtosis05', 'rw_kurtosis06', 'rw_kurtosis07',
    'rw_kurtosis08', 'rw_kurtosis09', 'rw_kurtosis10', 'avg_moving_speed01', 'avg_moving_speed02',
    'avg_moving_speed03', 'avg_moving_speed04', 'avg_moving_speed05', 'avg_moving_speed06',
    'avg_moving_speed07', 'avg_moving_speed08', 'avg_moving_speed09', 'avg_moving_speed10',
    'time_moving01', 'time_moving02', 'time_moving03', 'time_moving04', 'time_moving05',
    'time_moving06', 'time_moving07', 'time_moving08', 'time_moving09', 'time_moving10']

    theta_stats_list = []
    for i in gf.tau_range:
        for j in gf.interval_range:
            theta_stats_list.append( 'mean_theta_' + str(i) + '_' + str(j))
            theta_stats_list.append( 'min_theta_' + str(i) + '_' + str(j) )
            theta_stats_list.append( 'max_theta_' + str(i) + '_' + str(j) )

    turn_stats_list = []
    for i in gf.tau_range:
        for j in gf.interval_range:
            turn_stats_list.append( 'p_rturn_' + str(i) + '_' + str(j) )

    autocorr_stats_list = []
    for i in range(1, rwf.autocorr_max_tau):
        autocorr_stats_list.append('autocorr_' + str(i))

    for i in autocorr_stats_list:
        motility_header.append(i)
    for i in turn_stats_list:
        motility_header.append(i)
    for i in theta_stats_list:
        motility_header.append(i)


    with open(output_file_path, 'wb') as out_file:
        selectric = csv.writer(out_file, delimiter = ',', quoting=csv.QUOTE_NONE, quotechar='')
        selectric.writerow( motility_header )
        i = 0
        while i < len(merged_list):
            selectric.writerow( merged_list[i] )
            i += 1

    print 'Wrote ', output_dir, output_name


#------------------------------
# PICKLES FOR AREA ANALYSIS
#------------------------------

# Pickle cell_ids to be used by area_changes.py
# Uses sobel check from above to name output file

def pickle_cell_ids( cell_ids, sobel, output_dir ):
    if sobel == True:
        pickle_output = (output_dir + 'centroid_paths_sobel.pickle')
    else:
        pickle_output = (output_dir + 'centroid_paths.pickle')

    pickle.dump( cell_ids, file(pickle_output, 'w') )
    print "Pickeled cell_ids for ", output_dir

def pickle_removed_ids( removed_ids, sobel, output_dir ):
    if sobel == True:
        pickle_output = (output_dir + 'removed_ids_sobel.pickle')
    else:
        pickle_output = (output_dir + 'removed_ids.pickle')

    pickle.dump( removed_ids, file(pickle_output, 'w') )
    print "Pickled removed_ids for ", output_dir



from __future__ import division
from hmtools import dedupe
from hmstats import distance, average_xy
import sys

'''
#---------------------------
# MODULE CONTENTS
#---------------------------
Classes to/for:
1. Establish cell paths from centroid locations and time points
2. Sanity checking of the resulting cell paths
3. Identify colliding cells, remove one or both colliders from analysis

'''

#---------------------------
# ESTABLISH CELL PATHS
#---------------------------
# WIP
class CellPaths:
    '''
    CellPaths accepts either a list of lists of tuples containing centroid
    XY coordinates for each object in a frame (centroid_arrays)
    OR
    a dict keyed by unique cell identifiers
    (i.e. 0, 1, 2) with values as a list of tuples containg a single cell's XY
    coordinates, each tuple representing a given timepoint

    If an array of centroids is provided, CellPaths uses Heteromotility's
    built in tracking algorithm
    A dict of lists would be provided if another tracking algorithm is desired

    centroid_arrays = [
                        [(x,y), (x,y), (x,y), ...] # ea. sublist is a timepoint
                        [(x,y), (x,y), (x,y), # each tuple is one object's XY ]
                        ]

    cell_ids = {
                0 : [ (x1,y1), (x2,y2), (x3,y3), (x4,y4) ] # ea key is an object
                1 : [ (x1,y1), (x2,y2), (x3,y3), (x4,y4) ] # ea tuple is a timepoint
                }
    '''
    def __init__(self, centroid_arrays = [], cell_ids = {}, sanity_px = 200):
        if len(centroid_arrays) > 0:
            self.centroid_arrays = centroid_arrays
            self.cell_ids, self.removed_cells, self.points_corrected = self.establish_cell_paths(sanity_px)
        elif len(cell_ids) > 0:
            self.cell_ids, self.removed_cells, self.points_corrected = self.sanity_on_provided_ids(cell_ids, sanity_px)
        else:
            print "CellPaths requires an array of centroids or provided cell_ids"
            sys.exit()

    # assign each object in t0 centroids array a key in cell_ids dict
    # keys are just iterated integers, could do something fancier
    # if you need to for any reason
    def assign_cell_ids(self):
        cell_ids = {}
        i = 0
        for coor in self.centroid_arrays[0]:
            cell_ids[i] = []
            cell_ids[i].append(coor)
            i = i + 1
        return cell_ids

    # for each unique cell in cell_ids, find the closest coordinate to
    # the previous value in the next centroid array and append it
    # NOTE:
    # This assumes fairly limited cell motility b/w frames
    # and very little cell::cell contact

    def establish_cell_paths(self, sanity_px):
        cell_ids = self.assign_cell_ids()
        for array in self.centroid_arrays[1:]:
            for u in cell_ids:
                dists = {}
                for coor in array:
                    d = distance( coor, cell_ids[u][-1] )
                    dists[d] = coor
                closest_coor = dists[ min(dists) ]
                cell_ids[u].append(closest_coor)

        # Sanity Check
        cell_ids, remove, points_corrected = self.sanity_check(cell_ids, sanity_px)
        # Remove Insane
        cell_ids, removed = self.remove_cells(cell_ids, remove)

        return cell_ids, removed, points_corrected

    def sanity_on_provided_ids(self, cell_ids, sanity_px):
        cell_ids, remove, points_corrected = self.sanity_check(cell_ids, sanity_px)
        cell_ids, removed = self.remove_cells(cell_ids, remove)

        return cell_ids, removed, points_corrected

    #---------------------------
    # SANITY TESTING OF PATHS
    #---------------------------

    # Find points that move too quickly
    # (likely not segmented in one frame)
    # Correct for this by interpolating from the previous sensible point
    # and the next sensible point
    # Outputs a corrected cell_ids dict, a list of cells to remove,
    # and a list of lists with the points that were corrected

    def sanity_check(self, cell_ids, sanity_px):
        points_corrected = []
        remove = []
        for u in cell_ids:
            cell = cell_ids[u]

            for coor in cell[1:-4]: # exclude the first & last 4 coordinate
                previous_coor = cell[ ( cell.index(coor) - 1 ) ]
                d = distance( coor, previous_coor )
                if d > sanity_px:
                    i = 1
                    fix = False
                    next_coor = cell[ (cell.index(coor) + i) ]
                    while i < 4:
                        d2 = distance( previous_coor, next_coor )
                        if d2 < sanity_px:
                            fixed_coor = average_xy( previous_coor, next_coor )
                            cell_ids[u][ cell.index(coor) ] = fixed_coor
                            correction = [ coor, fixed_coor, previous_coor, next_coor ]
                            points_corrected.append(correction)
                            i = 4
                            fix = True
                        else:
                            i += 1
                    if fix == False:
                        remove.append(u)
                    else:
                        pass
                else:
                    pass

        # Remove duplicate removal marks from a single cell
        remove = dedupe(remove)

        return cell_ids, remove, points_corrected

    def remove_cells(self, cell_ids, to_remove):
        removed = {}
        for u in to_remove:
            removed[u] = cell_ids.pop(u, None)

        return cell_ids, removed


#------------------------------
# CHECK FOR REMAINING CELLS
#------------------------------

# Checks to see if any cells remain after removing cells
# that fail the sanity check
# If no cells are left, exits the script gracefully
# without pickling cell_ids or removed_ids

def check_remaining_cells( cell_ids ):

    if len(cell_ids) > 0:
        print len(cell_ids), ' cells remain after sanity checking \n'
    else:
        print "No cells remain after sanity checking \n"
        sys.exit()

#------------------------------
# CHECK TWO CELLS COLLIDING
#------------------------------


# Detects collisions of cells, and records them in a dict collider_log
# Stores all seen positions and cooresponding cells in seen = {}

# seen = { cell1 : coor, cell2 : coor, ... }

# When a collision is found, adds the colliding cells and coor to
# collider log

# collider_log = { t1: [ [cell1, cell2, coor], [cell3, cell4, coor]
#                   t2: ...                                           }

def find_colliders(cell_ids):

    collider_log = {}
    colliding_cells_log = {}
    seen_log = {}
    t = 0
    first_key = cell_ids.keys()[0]
    while t < len( cell_ids[ first_key ] ):
        seen = {}
        colliders = []
        colliding_cells = []

        for u in cell_ids:
            coor = cell_ids[u][t]
            if coor in seen.values():

                for prev_cell, prev_coor in seen.iteritems():
                    if coor == prev_coor:
                        colliders.append( [ (prev_cell, u) , coor] )
                        colliding_cells.append(u)
                        colliding_cells.append(prev_cell)
                    else:
                        pass
                seen[u] = coor
            else:
                seen[u] = coor

        seen_log[t] = seen
        collider_log[t] = colliders
        colliding_cells_log[t] = dedupe(colliding_cells)
        prop_colliders = float(len(dedupe(colliding_cells))) / float(len(cell_ids))

        t += 1

    return collider_log, prop_colliders

# Takes collider_log, checks to see if collisions
# are longer than a stringencey coeff.
# If not, marks cells as persistent colliders

def find_persistent_colliders(collider_log, stringency_coeff):

    t = 0
    persistent_colliders = []
    temp_colliders = []
    while t < (len(collider_log) - 30):
        timepoint = collider_log[t]
        for group in timepoint:
            collide_pair = group[0]
            future_pairs = []
            for i in range(1,31):
                future = collider_log[ t + i ]
                for future_group in future:
                    future_pairs.append( future_group[0] )

            if future_pairs.count( collide_pair ) > stringency_coeff:
                persistent_colliders.append( collide_pair )
            else:
                temp_colliders.append( collide_pair )
        t += 1

    persistent_colliders = dedupe(persistent_colliders)
    temp_colliders = dedupe(temp_colliders)

    return persistent_colliders, temp_colliders

# Takes persistent colliders & uses removal function
# from sanity testing to remove from cell_ids

def remove_both_colliders(persistent_colliders, cell_ids):

    to_remove = []
    for collide_pair in persistent_colliders:
        to_remove.append(collide_pair[0])
        to_remove.append(collide_pair[1])

    to_remove = dedupe(to_remove)
    to_remove.sort()

    cell_ids, collider_ids = remove_cells(cell_ids, to_remove)
    return cell_ids, collider_ids

def remove_one_collider(persistent_colliders, cell_ids):

    to_remove = []
    for collide_pair in persistent_colliders:
        to_remove.append(collide_pair[0])

    to_remove = dedupe(to_remove)
    to_remove.sort()

    cell_ids, collider_ids = remove_cells(cell_ids, to_remove)
    return cell_ids, collider_ids

def check_colliding_cells( cell_ids ):

    if len(cell_ids) > 0:
        print len(cell_ids), ' cells remain after collider checking \n'
    else:
        print "No cells remain after collider checking \n"
        sys.exit()



from hmstats import GeneralFeatures, MSDFeatures, RWFeatures
from hmtrack import CellPaths
from hmio import *
from hmtools import *

class DetailedBalance:
    '''
    Functions to split cell path's into discrete segments and calculate
    motility statistics on these sub-segments,
    such that 'motility state' transitions may be assessed within a single cell
    and the relevant timescale for 'motility state' defintion may be defined

    '''
    def __init__(self, cell_ids, min_split = 20):
        self.multi_split = self.multi_tau_split(cell_ids, tau_min = min_split)

    def split_ids(self, cell_ids, tau):
        '''
        Splits cell paths into multiple discrete segments of length tau
        Returns a dict of lists where each list is a subpath for easy stat
        calculations with existing Heteromotility infrastructure

        Parameters
        ----------------------
        cell_ids : dict of lists, ea. list containing tuples of XY coordinates
        tau : time lag, size of each split path

        Returns
        ----------------------
        split_ids :
        dict of lists, keyed by "cell_id-subpath_id"
        '''

        split_ids = {}
        n = len(cell_ids[ cell_ids.keys()[0] ])
        num_subseries = int(n/tau)
        for u in cell_ids:
            path = cell_ids[u]
            for i in range(0, num_subseries):
                sub = path[ i*tau : (i*tau) + tau ]
                split_ids[str(u) + '-' + str(i)] = sub

        return split_ids

    def multi_tau_split(self, cell_ids, tau_min = 20):
        '''
        Makes dict keyed by tau with split_ids for a range of time lags

        Parameters
        ----------------------
        cell_ids : dict of lists, ea. list containing tuples of XY coordinates
        tau_min : minimum time lag to consider

        n.b. max_tau is internally set at 0.5*len(time_series)

        Returns
        ----------------------
        '''
        tau_max = len( cell_ids[ cell_ids.keys()[0] ] )
        tau_max = int(tau_max / 2)
        tau_range = range( tau_min, tau_max )
        multi_split = {}
        for tau in tau_range:
            multi_split[tau] = self.split_ids(cell_ids, tau)

        return multi_split

    def split_id_features(self, multi_split, output_dir):
        '''
        Calculates motility features for subsegments of cell
        paths. Exports data as 'motility_statistics_split_$TAU.csv'
        where $TAU is the size of the segments being considered

        Parameters:
        multi_split : dict keyed by tau, containing cell_id style dicts
        output_dir : directory for output files
        '''

        for tau in multi_split:
            print tau
            split_ids = multi_split[tau]
            scp = CellPaths(cell_ids = split_ids)
            sgf = GeneralFeatures(cell_ids = split_ids, move_thresh = 10)
            smsdf = MSDFeatures(cell_ids = split_ids)
            srwf = RWFeatures(cell_ids = split_ids, gf= sgf)

            ind_outputs = single_outputs_list(scp.cell_ids, sgf, srwf, smsdf, output_dir)
            merged_list = make_merged_list(ind_outputs, sgf, srwf)
            output_name = 'motility_statistics_split_' + str(tau) + '.csv'
            write_motility_stats(output_dir, output_name, sgf, srwf, merged_list)


        return merged_list



#!/usr/bin/env python
from __future__ import division

from hmtools import *
from hmstats import GeneralFeatures, MSDFeatures, RWFeatures
from hmtrack import *
from hmio import *
import hmtests
import csv
import glob
import sys
import argparse
#import matplotlib.pyplot as plt
import pickle
import numpy as np
np.seterr(all='raise')

##########################
# TO ADD
##########################

# 25th & 75th %tile speeds with variable time interval
# Complete Test Suite
# Fractal Brownian Motion Simulation

##########################
# TO TEST
##########################

# general stats features


'''
Finds the nearest centroid to another centroid
Computes the distance and direction traveled

Assumes that objects detected at t0 are cells
If no object is found in a reasonable distance for two frames,
the object is considered an artifact and deleted

#-------------------------
# USAGE:
#-------------------------

heteromotility.py /input_dir/ /output_dir/

optional:
--sanity N # number of px to be considered 'sane' movement
--seg sobel # for sobel based segmentation
--exttrack /path_to_pickled/cell_ids.pickle

To use external tracker:
heteromotility.py ./ ./ --exttrack /path_to_pickled/cell_ids.pickle

'''
def main():

    # Parse CLI args
    parser = argparse.ArgumentParser('Calculate motility features from cell locations or cell paths')
    parser.add_argument('input_dir', action='store', default = ['./'], nargs = 1, help = "path to input directory of CSVs")
    parser.add_argument('output_dir', action='store', default = ['./'], nargs = 1, help = "directory for motility_statistics.csv export")
    parser.add_argument('--sanity', default = 200, help = "integer [px] determining the maximum sane movement of an object")
    parser.add_argument('--move_thresh', default = 10, help = "threshold to use when determining a cell is moving [px]")
    parser.add_argument('--seg', action = 'store', default = ['otsu'], nargs = 1, help = "name of an alternate seg method being used")
    parser.add_argument('--exttrack', action = 'store', default = [False], nargs = 1, help = "specifies external tracking algo, provide location of cell_ids.pickle")
    args = parser.parse_args()

    #------------------------------
    # IMPORT CENTROIDS FROM CSV
    #------------------------------
    input_dir = args.input_dir[0] + '*centroids*.csv'
    output_dir = args.output_dir[0]
    sanity = int(args.sanity)
    move_thresh = int(args.move_thresh)
    seg = args.seg[0]
    exttrack = args.exttrack[0]

    # Imports a directory of CSVs with centroid positions
    # Creates list of lists, each internal list == one time points
    # inner lists contain tuples with XY coors of centroid locations

    if exttrack == False:
        centroid_arrays = import_centroids(input_dir)
    else:
        pass

    #------------------------
    # ESTABLISH OBJECT PATHS
    #------------------------

    # See hmtrack.CellPaths for tracking implementation
    # if exttrack has specific an external pickle, load that as the cell_ids instead
    if exttrack == False:
        cp = CellPaths(centroid_arrays = centroid_arrays, sanity_px = sanity)
        cell_ids = cp.cell_ids
    else:
        cp = CellPaths( cell_ids = pickle.load( file(exttrack) ), sanity_px = sanity )
        cell_ids = cp.cell_ids

    #------------------------------
    # CHECK FOR REMAINING CELLS
    #------------------------------

    # Checks to see if any cells remain after removing cells
    # that fail the sanity check
    # If no cells are left, exits the script gracefully
    # without pickling cell_ids or removed_ids

    check_remaining_cells(cp.cell_ids)

    #------------------------------
    # CHECK TWO CELLS COLLIDING
    #------------------------------


    # Detects collisions of cells, handles in a selected way
    collider_log, prop_colliders = find_colliders(cp.cell_ids)

    print "Proportion of colliding cells = ", prop_colliders

    # Takes collider_log, checks to see if collisions
    # are longer than a stringencey coeff.
    # If not, marks cells as persistent colliders

    #persistent_colliders, temp_colliders = find_persistent_colliders(collider_log, 20)

    # Takes persistent colliders & uses removal function
    # from sanity testing to remove from cell_ids

    #cell_ids, collider_ids = remove_both_colliders(persistent_colliders, cell_ids)
    #cell_ids, collider_ids = remove_one_collider(persistent_colliders, cell_ids)
    #cell_ids, collider_ids = remove_cells(cell_ids, remove_colliders)
    #check_colliding_cells(cell_ids)

    #write_prop_colliders(prop_colliders, output_dir)

    #------------------------------
    # CALCULATE MOTILITY STATISTICS
    #------------------------------

    gf = GeneralFeatures(cp.cell_ids, move_thresh = move_thresh)

    #--------------------------------
    #     MEAN SQUARE DISPLACEMENT
    #--------------------------------

    msdf = MSDFeatures(cp.cell_ids)

    #--------------------------------
    # RANDOM WALK MODELING
    #--------------------------------
    # Compares each cell's path to a random walk using simulations to estimate
    # linearity and net_distance of comparable random walks
    # Compares to kurtosis of random walk displacement distribution (Rayleigh)

    rwf = RWFeatures(cp.cell_ids, gf)

    #------------------------------
    # WRITE STATISTICS TO CSV
    #------------------------------

    # Checks to see if non-standard segmentation is being used
    # Will output file with an altered name if yes
    def check_flags( flags ):
        if 'sobel' in flags:
            output_name = 'motility_statistics_sobel.csv'
            sobel = True
        else:
            output_name = 'motility_statistics.csv'
            sobel = False

        return output_name, sobel

    output_name, sobel = check_flags( seg )

    ind_outputs = single_outputs_list(cp.cell_ids, gf, rwf, msdf, output_dir)
    merged_list = make_merged_list(ind_outputs, gf, rwf)
    write_motility_stats(output_dir, output_name, gf, rwf, merged_list)


    #------------------------------
    # PICKLES FOR AREA ANALYSIS
    #------------------------------

    # Pickle cell_ids to be used by area_changes.py
    pickle_cell_ids( cp.cell_ids, sobel, output_dir )
    pickle_removed_ids( cp.removed_cells, sobel, output_dir )

    #------------------------------
    # Run Unit Tests
    #------------------------------

    print "test_sanity = ", hmtests.test_sanity()
    print "test_removal = ", hmtests.test_removal()

if __name__ == "__main__":
    main()



'''
#---------------------------
# MODULE CONTENTS
#---------------------------
Functions to/for:
Manipulate/shape data structures

'''

def dict2array(d):
    # takes dict of lists
    # d = {0: [a1, a2, a3], 1: [b1,b2,b3], 2: [...]}
    # merges into list of lists
    # output = [ [a1,a2,a3], [], [] ]
    output = []
    for u in d:
        output.append(list(d[u]))
    return output

def dictofdict2array(top_dict):
	# Takes dict of
	# dict_of_dict = { a: {x1 : a1, x2 : a2, x3 : a3},
	#				   b: {x1 : b1, x2 : b2, x3 : b3},
	#				   c: {x1 : c1, x2 : c2, x3 : c3} }
	# Outputs list of lists
	# output = [ [a1,b1,c1], [a2,b2,c2]... ]
	output = []
	i = 0
	while i < len( top_dict[ top_dict.keys()[0] ] ):
		row = []
		for key1 in top_dict:
			row.append( top_dict[key1][ top_dict[key1].keys()[i] ] )
		output.append(row)
		i += 1
	return output

def tripledict2array(top_dict):
	output = []
	j = 0
	while j < len( top_dict[ top_dict.keys()[0] ][ top_dict[ top_dict.keys()[0] ].keys()[0] ] ):
		row = []
		for key1 in top_dict:
			for key2 in top_dict[key1]:
				if type(top_dict[key1][key2][ top_dict[key1][key2].keys()[j] ]) == list:
					for item in top_dict[key1][key2][ top_dict[key1][key2].keys()[j] ]:
						row.append(item)
				else:
					row.append( top_dict[key1][key2][ top_dict[key1][key2].keys()[j] ] )
		output.append(row)
		j += 1
	return output

# Super fast deduping of lists, preserves order
# Credit to Peterbe
# http://www.peterbe.com/plog/uniqifiers-benchmark
def dedupe(seq, idfun=None):
   # order preserving
   if idfun is None:
       def idfun(x): return x
   seen = {}
   result = []
   for item in seq:
       marker = idfun(item)
       # in old Python versions:
       # if seen.has_key(marker)
       # but in new ones:
       if marker in seen: continue
       seen[marker] = 1
       result.append(item)
   return result

# Takes a list of lists of lists [ [ [], ... ], [ [], ... ], ...]
# Returns list of lists, with each nth list containing the values of the
# nth tertiary lists merged together
import itertools
def merge_flat_lists(lists):
	# lists = [
    #           [ [...], [...], [...] ],
    #           [ [...], [...], ... ], ...
    #                                        ]
	merged_list = []
	i = 0
	while i < len(lists[0]):
		tmp_list = []
		for l in lists:
			tmp_list.append(l[i])

		tmp_merged = list( itertools.chain( *tmp_list ) )
		merged_list.append(tmp_merged)
		i += 1

	# merged_list = [ [all vals for one cell], [...], ... ]
	return merged_list

def single_outputs_list(cell_ids, gf, rwf, msdf, output_dir):
    # Creates a list of lists for writing out statistics
    # Ea. internal list is a single cell's stats
    output_list = []
    for cell in cell_ids:
        output_list.append([ output_dir, cell, gf.total_distance[cell], gf.net_distance[cell],
                            gf.linearity[cell], gf.spearmanrsq[cell], gf.progressivity[cell], gf.max_speed[cell],
                            gf.min_speed[cell], gf.avg_speed[cell], msdf.alphas[cell], rwf.hurst_RS[cell], rwf.nongaussalpha[cell],
                            rwf.disp_var[cell], rwf.disp_skew[cell], rwf.diff_linearity[cell], rwf.diff_net_dist[cell] ])

    return output_list

def make_merged_list(ind_outputs, gf, rwf):
    autocorr_array = dict2array(rwf.autocorr)
    diff_kurtosis_array = dictofdict2array(rwf.diff_kurtosis)
    avg_moving_speed_array = dictofdict2array( gf.avg_moving_speed )
    time_moving_array = dictofdict2array( gf.time_moving )
    turn_list = tripledict2array(gf.turn_stats)
    theta_list = tripledict2array(gf.theta_stats)

    merged_list = merge_flat_lists([ind_outputs, diff_kurtosis_array, avg_moving_speed_array, time_moving_array, autocorr_array, turn_list, theta_list])
    return merged_list



from __future__ import division
import random
from hmstats import GeneralFeatures, RWFeatures, MSDFeatures
from hmtrack import CellPaths
from hmtools import dictofdict2array, tripledict2array, merge_flat_lists


'''
#---------------------------
# MODULE CONTENTS
#---------------------------
Classes to generate simulated data sets for analysis, including
1. Random walk simulations
2. Biased random walk simulations

#---------------------------
# To Do
#---------------------------

3. Levy flight simulations
4. Fractal brownian motion simulations


'''


class RWSims:
    def __init__(self, obj = 100, length = 100):
        self.rw_paths = self.rwsim(obj, length)
        self.brw_paths = self.biased_rwsim(obj, length)

    def random_walk(self, N, speed_mu = 7.0, speed_sigma = None, origin = (0,0)):
        import numpy as np
        '''
        Models random walk given an origin, length N, step size : rate
        abbreviated from implementation in hmstats.RWFeatures

        Parameters
        -------------
        origin : origin point, default = (0,0)
        N : length of walk to model
        rate : step size to model

        Returns
        -------------
        model_path : list containing sequential tuples of XY coordinates

        Notes
        -------------
        step_size = rate = c = sqrt(x_step^2 + y_step^2) = sqrt(2*step^2)
        x_step = y_step = rate/sqrt(2)
        (Random Walks in Biology, 1992, Howard C. Berg)
        '''

        model_path = [ origin ]
        if speed_sigma == None:
            speed_sigma = 0.2*speed_mu
        i = 0
        while i < N:
            rate = round(np.random.normal(speed_mu, speed_sigma), 3)
            vectors = [ (0, rate), (0, -rate), (rate, 0), (-rate, 0) ]
            walk = np.random.random()
            if 0 <= walk < 0.25:
                step = vectors[0]
            elif 0.25 <= walk < 0.50:
                step = vectors[1]
            elif 0.50 <= walk < 0.75:
                step = vectors[2]
            else:
                step = vectors[3]
            new_x = model_path[-1][0] + step[0]
            new_y = model_path[-1][1] + step[1]
            model_path.append( (new_x, new_y) )

            i += 1

        return model_path

    def rwsim(self, obj, length, speed_range = range(5, 16)):
        '''
        Generates simulated object paths using an unbiased random walk model

        Parameters
        ------------
        obj : integer specifying the number of object paths to be simulated
        length : integer specifying the length of each path to be simulated
        speed_range : tuple containing integers of step sizes to use when
        building models

        Returns
        ------------
        rw_paths : dict keyed by speed, containing dicts keyed by obj id,
        containing lists of tuples, each tuple
        specifying the XY position of a simulated object at a given timepoint
            rw_paths =
            {
            speed_i : {0: [ (x1,y1), (x2,y2)...(x_n, y_n) ]
                        1: [ (x1,y1), (x2,y2)...(x_n, y_n) ]
                        ...},
            speed_i+1 : ...,
            ...
            }
        '''

        rw_paths = {}

        for speed in speed_range:
            rw_paths[speed] = {}
            i = 0
            while i < obj:
                model_path = self.random_walk(N = length, speed_mu = speed, origin = (0,0))
                rw_paths[speed][i] = model_path
                i += 1

        return rw_paths

    def biased_random_walk(self, N, speed_mu = 7.0, speed_sigma = None, bias_prop = 0.5 , origin = (0,0)):
        import numpy as np
        '''
        * Models a set of random walks biased to move in one direction
        * Randomly selects in which direction to bias movement
        * Object will move in the direction of bias a proportion of the time
        specified in bias_prop

        Parameters
        ------------
        N : length of walk to be modeled
        rate : step size to be modeled
        bias : proportion of the time an object moves in the direction of bias
        origin : origin point for the start of the walk to be modeled

        Returns
        ------------
        model_path : list of tuples, each containing an XY point

        '''
        model_path = [origin]
        if speed_sigma == None:
            speed_sigma = 0.2*speed_mu
        bias_choice = random.randrange(start = 0, stop = 4, step = 1)
        i = 0
        while i < N:
            walk = np.random.random()
            rate = round(np.random.normal(speed_mu, speed_sigma), 3)
            vectors = [ (0, rate), (0, -rate), (rate, 0), (-rate, 0) ]
            bias = vectors[bias_choice]
            vectors.pop(bias_choice)
            unbiased_prop = 1-bias_prop
            break1 = bias_prop + (unbiased_prop/3)
            break2 = bias_prop + 2*(unbiased_prop/3)
            if walk <= bias_prop:
                step = bias
            elif bias_prop < walk <= break1:
                step = vectors[0]
            elif break1 < walk <= break2:
                step = vectors[1]
            else:
                step = vectors[2]

            new_x = model_path[-1][0] + step[0]
            new_y = model_path[-1][1] + step[1]
            model_path.append( (new_x, new_y) )
            i += 1

        return model_path

    def biased_rwsim(self, obj, length, speed_range = range(5,16)):
        '''
        Generates simulated object paths using a biased random walk model

        Parameters
        ------------
        obj : integer specifying the number of object paths to be simulated
        length : integer specifying the length of each path to be simulated

        Returns
        ------------
        brw_paths : dict keyed by obj id, containing lists of tuples, each tuple
        specifying the XY position of a simulated object at a given timepoint
            rw_paths = {
            0: [ (x1,y1), (x2,y2)...(x_n, y_n) ]
            1: [ (x1,y1), (x2,y2)...(x_n, y_n) ]
            ...
            }
        '''

        brw_paths = {}

        for speed in speed_range:
            brw_paths[speed] = {}
            i = 0
            while i < obj:
                model_path = self.biased_random_walk(N = length, speed_mu = speed, bias_prop = 0.5, origin = (0,0))
                brw_paths[speed][i] = model_path
                i += 1

        return brw_paths

class LevySims:
    def __init__(self, obj = 100, length = 100):
        self.power_paths = self.power_sim(obj, length)
        self.levy_paths = self.levy_sim(obj, length)
        self.fly = levy_flier(N = 100)

    def power_flier( self, N, x_min = 0.1, alpha = 2.000, scale_coeff=50, origin = (0,0) ):
        '''
        Defines a levy flight path beginning at the origin, taking N steps,
        with step sizes pulled from a power law distribution

        Directions are chosen randomly, as in a random walk
        Step sizes however are chosen according to the distribution:

            P(l_j) ~ l_j**(-a)
            where a is a parameter in the range 1 < u <= 3

        Defaults to a = 2 as the optimal parameter for foraging behavior
        see : Viswanathan 1999, Nature

        Obtaining Levy distributed random variables
        -------------
        To transform uniformly distributed random variable
        into another distribution, use inverse cum. dist. func. (CDF)

        if F is CDF corresponding to probability density of variable f and u
        is a uniformly distributed random variable on [0,1]

        x = F^-1(u)
        for pure power law distribution P(l) = l**-a
        F(x) = 1 - ( x / x_min )
        where x_min is a lower bound on the random variable

        F^-1(u) = x_min(1 - u)^(-1/a)

        See: (Devroye 1986, Non-uniform random variable generation)

        or

        X = F^-1(U) = c / ( phi^-1(1-U/2) )**2 + mu

        c : scaling parameter
        Phi(x) : CDF of Gaussian distribution
        mu : location

        for a pure Levy distribution

        see http://www.math.uah.edu/stat/special/Levy.html

        Parameters
        -------------
        N : number of steps in the flight
        step_min : integer value, minimum step size
        origin : initial site of walk
        u = parameter for probability distribution of step sizes

        Returns
        -------------
        model_path : a list containing tuples of XY coordinates
        '''

        model_path = [ origin ]
        i = 0
        while i < N:
            rate = ( x_min*( 1-random.random() )**(-1/alpha) ) * scale_coeff
            vectors = [ (0, rate), (0, -rate), (rate, 0), (-rate, 0) ]
            walk = random.random()
            if 0 <= walk < 0.25:
                step = vectors[0]
            elif 0.25 <= walk < 0.50:
                step = vectors[1]
            elif 0.50 <= walk < 0.75:
                step = vectors[2]
            else:
                step = vectors[3]

            new_x = model_path[-1][0] + step[0]
            new_y = model_path[-1][1] + step[1]
            model_path.append( (new_x, new_y) )
            i += 1

        return model_path

    def power_sim(self, obj, length, scale_range = range(40,60)):
        '''
        Generates simulated object paths using a levy flight model with
        a power law distribution of displacements

        Parameters
        ------------
        obj : integer specifying the number of object paths to be simulated
        length : integer specifying the length of each path to be simulated

        Returns
        ------------
        levy_paths : dict keyed by obj id, containing lists of tuples, each tuple
        specifying the XY position of a simulated object at a given timepoint
            levy_paths = {
            0: [ (x1,y1), (x2,y2)...(x_n, y_n) ]
            1: [ (x1,y1), (x2,y2)...(x_n, y_n) ]
            ...
            }
        '''

        power_paths = {}

        for scale in scale_range:
            power_paths[scale] = {}
            i = 0
            while i < obj:
                model_path = self.power_flier(N = length, scale_coeff=scale)
                power_paths[scale][i] = model_path
                i += 1

        return power_paths

    def levy_flier(self, N, scale_coeff = 1, origin = (0,0)):
        '''
        Defines a levy flight path beginning at the origin, taking N steps,
        with step sizes pulled from a true Levy distribution

        scale = 2, median disp = 4.4
        scale = ~2.3 has a median displacement of 5 units

        Directions are chosen randomly, as in a random walk
        Step sizes however are chosen according to the distribution:

            P(l_j) ~ l_j**(-a)
            where a is a parameter in the range 1 < u <= 3

        Defaults to a = 2 as the optimal parameter for foraging behavior
        see : Viswanathan 1999, Nature

        Obtaining Levy distributed random variables
        -------------
        To transform uniformly distributed random variable
        into another distribution, use inverse cum. dist. func. (CDF)

        if F is CDF corresponding to probability density of variable f and u
        is a uniformly distributed random variable on [0,1]

        x = F^-1(u)
        for pure power law distribution P(l) = l**-a
        F(x) = 1 - ( x / x_min )
        where x_min is a lower bound on the random variable

        F^-1(u) = x_min(1 - u)^(-1/a)

        See: (Devroye 1986, Non-uniform random variable generation)

        or

        X = F^-1(U) = c / ( phi^-1(1-U/2) )**2 + mu

        c : scaling parameter
        Phi(x) : CDF of Gaussian distribution
        mu : location

        for a pure Levy distribution

        see http://www.math.uah.edu/stat/special/Levy.html

        Parameters
        -------------
        N : number of steps in the flight
        step_min : integer value, minimum step size
        origin : initial site of walk
        u = parameter for probability distribution of step sizes

        Returns
        -------------
        model_path : a list containing tuples of XY coordinates
        '''
        from scipy import stats.levy

        model_path = [ origin ]
        i = 0
        while i < N:
            rate = levy.rvs(scale=scale_coeff)
            vectors = [ (0, rate), (0, -rate), (rate, 0), (-rate, 0) ]
            walk = random.random()
            if 0 <= walk < 0.25:
                step = vectors[0]
            elif 0.25 <= walk < 0.50:
                step = vectors[1]
            elif 0.50 <= walk < 0.75:
                step = vectors[2]
            else:
                step = vectors[3]

            new_x = model_path[-1][0] + step[0]
            new_y = model_path[-1][1] + step[1]
            model_path.append( (new_x, new_y) )
            i += 1

        return model_path

    def levy_sim(self, obj, length, scale_range = range(1,10)):
        '''
        Generates simulated object paths using a levy flight model with a true
        Levy distribution of displacements

        Parameters
        ------------
        obj : integer specifying the number of object paths to be simulated
        length : integer specifying the length of each path to be simulated

        Returns
        ------------
        levy_paths : dict keyed by obj id, containing lists of tuples, each tuple
        specifying the XY position of a simulated object at a given timepoint
            levy_paths = {
            0: [ (x1,y1), (x2,y2)...(x_n, y_n) ]
            1: [ (x1,y1), (x2,y2)...(x_n, y_n) ]
            ...
            }
        '''

        levy_paths = {}

        for scale in scale_range:
            power_paths[scale] = {}
            i = 0
            while i < obj:
                model_path = self.levy_flier(N = length, scale_coeff=scale)
                power_paths[scale][i] = model_path
                i += 1

        return levy_paths

'''
class fBmsims:
    def __init__(self, obj = 100, length = 100):
        self.fbm_paths = self.fbm_sim(obj, length)

    def fractal_brownian( self, N, T, H = 0.5, origin = (0,0) ):
        if H > 1 or H < 0:
            return None

        r = np.zeros((n + 1, 1))
        r[0] = 1
        for i in range(1,n):
            r[i] = 0.5 * ( (i+1)**(2*H) - (2*i)**(2*H) + (i-1)**(2*H))
            r =
'''
#---------------------------
# Export Simulated Data
#---------------------------
'''
rws = RWSims(obj = 1000, length = 100)
lfs = LevySims(obj = 1000, length = 100)

for sim in [rws, lfs]:
    cp = CellPaths(cell_ids = rws)
    gf = GeneralFeatures(cell_ids = cp.cell_ids)
    msdf = MSDFeatures(cell_ids = cp.cell_ids)
    rwf = RWFeatures(cell_ids = cp.cell_ids, gf = gf)

    ind_outputs = single_outputs_list(cp.cell_ids, gf = gf, rwf = rwf, msdf = msdf, 'sim_output')
    diff_kurtosis_array = dictofdict2array(rwf.diff_kurtosis)
    avg_moving_speed_array = dictofdict2array( gf.avg_moving_speed )
    time_moving_array = dictofdict2array( gf.time_moving )
    turn_list = tripledict2array(gf.turn_stats)
    theta_list = tripledict2array(gf.theta_stats)
    merged_list = merge_flat_lists([ind_outputs, diff_kurtosis_array, avg_moving_speed_array, time_moving_array, turn_list, theta_list])
'''



#------------------------------
# PLOT CELL MOTILITY PATHS
#------------------------------

def plot_paths(cell_ids):
	for u in cell_ids:
		cell = cell_ids[u]
		x = []
		y = []
		for coor in cell:
			x.append( float(coor[0]) )
			y.append( float(coor[1]) )

	# Draws a scatter plot of x, y values for each path
	# Connects plots with lines
		plt.scatter(x, y)
		plt.plot(x, y)



#------------------------------
# FIT SPLINES TO CELL PATHS
#------------------------------

def fit_splines(cell_ids):
	splines = [] # list of np.arrays in tuple pairs (x_array, y_array)
	for u in cell_ids:
		cell = cell_ids[u]
		x = []
		y = []
		for coor in cell:
			x.append( float(coor[0]) )
			y.append( float(coor[1]) )

		# Calculate a range spline
		x = np.asarray(x)
		y = np.asarray(y)
		t = np.arange(x.shape[0], dtype=float)
		t /= t[-1]
		nt = np.linspace(0, 1, 100)
		x1 = interpolate.spline( t, x, nt )
		y1 = interpolate.spline( t, y, nt )

		# Calculate a distance spline
		t = np.zeros(x.shape)
		t[1:] = np.sqrt( (x[1:] - x[:-1])**2 + (y[1:] - y[:-1])**2 )
		t = np.cumsum(t)
		t /= t[-1]
		x2 = interpolate.spline(t, x, nt)
		y2 = interpolate.spline(t, y, nt)

		splines.append( (x1, y1 , x2, y2) )


	return splines, x1, y1, x2, y2



from setuptools import setup

setup(name='heteromotility',
      version='0.1',
      description='Quantitative analysis of biological motion',
      url='http://github.com/jacobkimmel/heteromotility',
      author='Jacob Kimmel',
      author_email='jacobkimmel@gmail.com',
      license='MIT',
      packages=['heteromotility'],
      install_requires = [
      'numpy',
      'scipy',
      'argparse',
      'statsmodels'
      ],
      entry_points = {
      'console_scripts': ['heteromotility = heteromotility.command_line:cli']
      },
      include_package_data = True,
      zip_safe=False)
